/*
 * scientific visualization api for direct framebuffer
 *
 * author: hackbard@hackdaworld.dyndns.org
 *
 */

#include <stdio.h>
#include <directfb.h>
#include "dfbapi.h"

/* two dimensional lattice */

int d2_lattice_init(int *argc,char **argv,d2_lattice *d2_l)
{
 DFBSurfaceDescription surface_dsc;
 DFBFontDescription font_dsc;
 
 DirectFBInit(argc,&argv);
 DirectFBCreate(&(d2_l->dfb));
 d2_l->dfb->SetCooperativeLevel(d2_l->dfb,DFSCL_FULLSCREEN);

 surface_dsc.flags=DSDESC_CAPS;
 surface_dsc.caps=DSCAPS_PRIMARY|DSCAPS_FLIPPING;
 d2_l->dfb->CreateSurface(d2_l->dfb,&surface_dsc,&(d2_l->p_surface));
 d2_l->p_surface->GetSize(d2_l->p_surface,&(d2_l->s_width),&(d2_l->s_height));
 
 font_dsc.flags=DFDESC_HEIGHT;
 font_dsc.height=d2_l->s_height/40; /* 40 ? */
 d2_l->font_h=font_dsc.height;
 d2_l->dfb->CreateFont(d2_l->dfb,FONT,&font_dsc,&(d2_l->font));
 d2_l->p_surface->SetFont(d2_l->p_surface,d2_l->font);

 d2_l->fakt_y=(d2_l->s_height-(2*Y_GAP))/(d2_l->max_y);
 d2_l->fakt_x=(d2_l->s_height-X_GAP)/(d2_l->max_x); /* bullshit, i can't imagine atm */
 d2_l->info_x=d2_l->fakt_x*d2_l->max_x+2*X_GAP;
 d2_l->info_y=Y_GAP;
 d2_l->info_w=d2_l->s_width-d2_l->info_x-2*X_GAP;
 d2_l->info_h=d2_l->max_y*d2_l->fakt_y;

 if((d2_l->s_height<(d2_l->max_y+2*Y_GAP)) || (d2_l->s_height<(d2_l->max_x+X_GAP)))
 {
  puts("resolution too low!");
  return -1;
 } else return 1;
}

int d3_lattice_init(int *argc,char **argv,d3_lattice *d3_l)
{
 DFBSurfaceDescription surface_dsc;
 DFBFontDescription font_dsc;
 
 DirectFBInit(argc,&argv);
 DirectFBCreate(&(d3_l->dfb));
 d3_l->dfb->SetCooperativeLevel(d3_l->dfb,DFSCL_FULLSCREEN);

 surface_dsc.flags=DSDESC_CAPS;
 surface_dsc.caps=DSCAPS_PRIMARY|DSCAPS_FLIPPING;
 d3_l->dfb->CreateSurface(d3_l->dfb,&surface_dsc,&(d3_l->p_surface));
 d3_l->p_surface->GetSize(d3_l->p_surface,&(d3_l->s_width),&(d3_l->s_height));
 
 font_dsc.flags=DFDESC_HEIGHT;
 font_dsc.height=d3_l->s_height/40; /* 40 ? */
 d3_l->font_h=font_dsc.height;
 d3_l->dfb->CreateFont(d3_l->dfb,FONT,&font_dsc,&(d3_l->font));
 d3_l->p_surface->SetFont(d3_l->p_surface,d3_l->font);

 d3_l->fakt_y=(d3_l->s_height-(3*Y_GAP))/(d3_l->max_z+d3_l->max_y);
 d3_l->fakt_x=(d3_l->s_width-(3*X_GAP))/(d3_l->max_x+d3_l->max_y);
 d3_l->info_x=d3_l->fakt_x*d3_l->max_x+(2*X_GAP);
 d3_l->info_y=d3_l->fakt_y*d3_l->max_z+(2*Y_GAP);
 d3_l->info_w=d3_l->fakt_x*d3_l->max_y;
 d3_l->info_h=d3_l->fakt_y*d3_l->max_y;

 return 1;
}

int d2_lattice_release(d2_lattice *d2_l)
{
 d2_l->font->Release(d2_l->font);
 d2_l->p_surface->Release(d2_l->p_surface);
 d2_l->dfb->Release(d2_l->dfb);

 return 1;
}

int d3_lattice_release(d3_lattice *d3_l)
{
 d3_l->font->Release(d3_l->font);
 d3_l->p_surface->Release(d3_l->p_surface);
 d3_l->dfb->Release(d3_l->dfb);

 return 1;
}

int dx_lattice_get_color(unsigned char *status,unsigned char *r,unsigned char *g,unsigned char *b,unsigned char mode)
{
 *g=0;
 if(!mode)
 {
  if((*status)&RED)
  {
   *r=0xff;
   *b=0;
  } else
  {
   *r=0;
   *b=0xff;
  }
 } else if(mode==1)
 {
  *r=*status;
  *g=*status;
  *b=0xff;
 } else if(mode==2)
 {
  *r=*status;
  *b=0xff;
 }
 
 return 1;
}

int d2_lattice_draw(d2_lattice *d2_l,int x,int y,int arg_c,char **arg_v,unsigned char mode,int max_extra)
{
 int x_c,y_c;
 int i;
 unsigned char r,g,b,a;
 unsigned char foobar;

 a=0xff; /* no alpha blending */
 
 for(x_c=0;x_c<d2_l->max_x;x_c++)
 {
  for(y_c=0;y_c<d2_l->max_y;y_c++)
  {
   if(!mode) dx_lattice_get_color((*d2_l).status+x_c+y_c*d2_l->max_x,&r,&g,&b,mode);
   if(mode==1)
   {
    foobar=(unsigned char)(*(d2_l->extra+x_c+y_c*d2_l->max_x)*255/max_extra);
    dx_lattice_get_color(&foobar,&r,&g,&b,mode);
   }
   if(mode==2)
   {
    foobar=(*(unsigned char *)(d2_l->v_ptr+x_c+y_c*d2_l->max_x));
    dx_lattice_get_color(&foobar,&r,&g,&b,mode);
   }
   if(x_c==x && y_c==y)
   {
    r=0xff;
    g=0xff;
    b=0;
   }
   d2_l->p_surface->SetColor(d2_l->p_surface,r,g,b,a);
   d2_l->p_surface->DrawRectangle(d2_l->p_surface,
                                  x_c*d2_l->fakt_x+X_GAP,
                                  y_c*d2_l->fakt_y+Y_GAP,
                                  d2_l->fakt_x,d2_l->fakt_y);
  }
 }
 r=0xff;
 g=0xff;
 b=0;
 /* clear info box */
 d2_l->p_surface->SetColor(d2_l->p_surface,0,0,0,0);
 d2_l->p_surface->FillRectangle(d2_l->p_surface,
                                d2_l->info_x,d2_l->info_y,
                                d2_l->info_w,d2_l->info_h);
 d2_l->p_surface->SetColor(d2_l->p_surface,r,g,b,a);
 d2_l->p_surface->DrawRectangle(d2_l->p_surface,
                                d2_l->info_x,d2_l->info_y,
                                d2_l->info_w,d2_l->info_h);
 d2_l->p_surface->SetColor(d2_l->p_surface,0x80,0x80,0xff,0xff);
 for(i=1;i<=arg_c;i++)
 {
  d2_l->p_surface->DrawString(d2_l->p_surface,arg_v[i],-1,
                              d2_l->info_x+d2_l->fakt_x,
                              d2_l->info_y+d2_l->fakt_y+d2_l->font_h+(i-1)*2*d2_l->font_h,
                              DSTF_LEFT);
 }
 /* now we flip all to surface */
 d2_l->p_surface->Flip(d2_l->p_surface,NULL,0);
 
 return 1;
}

int d3_lattice_draw(d3_lattice *d3_l,int x,int y,int z,int arg_c,char **arg_v,unsigned char mode,int max_extra)
{
 int x_c,y_c,z_c;
 int i;
 unsigned char foobar;
 unsigned char r,g,b,a;

 a=0xff; /* no alpha blending */

 y_c=y;
 for(x_c=0;x_c<d3_l->max_x;x_c++)
 {
  for(z_c=0;z_c<d3_l->max_z;z_c++)
  {
   if(!mode) dx_lattice_get_color((*d3_l).status+x_c+y_c*d3_l->max_x+z_c*d3_l->max_x*d3_l->max_y,&r,&g,&b,mode);
   if(mode==1)
   {
    foobar=(unsigned char)(*(d3_l->extra+x_c+y_c*d3_l->max_x+z_c*d3_l->max_x*d3_l->max_y)*255/max_extra);
    dx_lattice_get_color(&foobar,&r,&g,&b,mode);
   }
   if(mode==2)
   {
    foobar=(*(unsigned char *)(d3_l->v_ptr+x_c+y_c*d3_l->max_x+z_c*d3_l->max_x*d3_l->max_y));
    dx_lattice_get_color(&foobar,&r,&g,&b,mode);
   }
   if(x_c==x && z_c==z)
   {
    r=0xff;
    g=0xff;
    b=0;
   }
   d3_l->p_surface->SetColor(d3_l->p_surface,r,g,b,a);
   d3_l->p_surface->DrawRectangle(d3_l->p_surface,
                                  x_c*d3_l->fakt_x+X_GAP,
                                  z_c*d3_l->fakt_y+Y_GAP,
                                  d3_l->fakt_x,d3_l->fakt_y);
  }
 }

 x_c=x;
 for(y_c=0;y_c<d3_l->max_y;y_c++) 
 {
  for(z_c=0;z_c<d3_l->max_z;z_c++)
  {
   if(!mode) dx_lattice_get_color((*d3_l).status+x_c+y_c*d3_l->max_x+z_c*d3_l->max_x*d3_l->max_y,&r,&g,&b,mode);
   if(mode==1)
   {
    foobar=(unsigned char)(*(d3_l->extra+x_c+y_c*d3_l->max_x+z_c*d3_l->max_x*d3_l->max_y)*255/max_extra);
    dx_lattice_get_color(&foobar,&r,&g,&b,mode);
   }
   if(mode==2)
   {
    foobar=(*(unsigned char *)(d3_l->v_ptr+x_c+y_c*d3_l->max_x+z_c*d3_l->max_x*d3_l->max_y));
    dx_lattice_get_color(&foobar,&r,&g,&b,mode);
   }
   if(y_c==y && z_c==z)
   {
    r=0xff;
    g=0xff;
    b=0;
   }
   d3_l->p_surface->SetColor(d3_l->p_surface,r,g,b,a);
   d3_l->p_surface->DrawRectangle(d3_l->p_surface,
                                  y_c*d3_l->fakt_x+d3_l->info_x,
                                  z_c*d3_l->fakt_y+Y_GAP,
                                  d3_l->fakt_x,d3_l->fakt_y);
  }
 }

 z_c=z;
 for(x_c=0;x_c<d3_l->max_x;x_c++)
 {
  for(y_c=0;y_c<d3_l->max_y;y_c++)
  {
   if(!mode) dx_lattice_get_color((*d3_l).status+x_c+y_c*d3_l->max_x+z_c*d3_l->max_x*d3_l->max_y,&r,&g,&b,mode);
   if(mode==1)
   {
    foobar=(unsigned char)(*(d3_l->extra+x_c+y_c*d3_l->max_x+z_c*d3_l->max_x*d3_l->max_y)*255/max_extra);
    dx_lattice_get_color(&foobar,&r,&g,&b,mode);
   }
   if(mode==2)
   {
    foobar=(*(unsigned char *)(d3_l->v_ptr+x_c+y_c*d3_l->max_x+z_c*d3_l->max_x*d3_l->max_y));
    dx_lattice_get_color(&foobar,&r,&g,&b,mode);
   }
   if(x_c==x && y_c==y)
   {
    r=0xff;
    g=0xff;
    b=0;
   }
   d3_l->p_surface->SetColor(d3_l->p_surface,r,g,b,a);
   d3_l->p_surface->DrawRectangle(d3_l->p_surface,
                                  x_c*d3_l->fakt_x+X_GAP,
                                  y_c*d3_l->fakt_y+d3_l->info_y,
                                  d3_l->fakt_x,d3_l->fakt_y);
  }
 }

 r=0xff;
 g=0xff;
 b=0;
 /* clear info box */
 d3_l->p_surface->SetColor(d3_l->p_surface,0,0,0,0);
 d3_l->p_surface->FillRectangle(d3_l->p_surface,
                                d3_l->info_x,d3_l->info_y,
                                d3_l->info_w,d3_l->info_h);
 d3_l->p_surface->SetColor(d3_l->p_surface,r,g,b,a);
 d3_l->p_surface->DrawRectangle(d3_l->p_surface,
                                d3_l->info_x,d3_l->info_y,
                                d3_l->info_w,d3_l->info_h);
 d3_l->p_surface->DrawLine(d3_l->p_surface,
                           d3_l->info_x+d3_l->info_w/2,
                           d3_l->info_y,
                           d3_l->info_x+d3_l->info_w/2,
                           d3_l->info_y+d3_l->info_h);
 d3_l->p_surface->SetColor(d3_l->p_surface,0x80,0x80,0xff,0xff);
 for(i=1;i<=arg_c;i++)
 {
  if(i<13)
  {
   d3_l->p_surface->DrawString(d3_l->p_surface,arg_v[i],-1,
                               d3_l->info_x+d3_l->fakt_x,
                               d3_l->info_y+2*d3_l->fakt_y+d3_l->font_h+(i-1)*d3_l->font_h,
                               DSTF_LEFT);
  } else
  {
   d3_l->p_surface->DrawString(d3_l->p_surface,arg_v[i],-1,
                               d3_l->info_x+d3_l->info_w/2+d3_l->fakt_x,
                               d3_l->info_y+2*d3_l->fakt_y+d3_l->font_h+(i%13-1)*d3_l->font_h,
                               DSTF_LEFT);
  }
 }

 /* now we flip all to surface */
 d3_l->p_surface->Flip(d3_l->p_surface,NULL,0);

 return 1;
}

int d2_event_init(d2_lattice *d2_l)
{
 d2_l->dfb->GetInputDevice(d2_l->dfb,DIDID_KEYBOARD,&(d2_l->keyboard));
 d2_l->keyboard->CreateEventBuffer(d2_l->keyboard,&(d2_l->k_buffer));
 
 return 1;
}

int d3_event_init(d3_lattice *d3_l)                                           
{                                                                             
 d3_l->dfb->GetInputDevice(d3_l->dfb,DIDID_KEYBOARD,&(d3_l->keyboard));
 d3_l->keyboard->CreateEventBuffer(d3_l->keyboard,&(d3_l->k_buffer));  

 return 1;                                                           
}         

int scan_event(d3_lattice *d3_l,int *x,int *y,int *z,int *q,int *esc,int *switchmode,int *bmp,int *ac_distr)
{
 DFBInputEvent ke;

 d3_l->k_buffer->WaitForEvent(d3_l->k_buffer);
 d3_l->k_buffer->GetEvent(d3_l->k_buffer,DFB_EVENT(&ke));
 if(ke.type==DIET_KEYPRESS)
 {
  if(ke.key_id==DIKI_LEFT && *x!=0) *x-=1;
  if(ke.key_id==DIKI_RIGHT && *x!=d3_l->max_x-1) *x+=1;
  if(ke.key_id==DIKI_UP && *y!=0) *y-=1;
  if(ke.key_id==DIKI_DOWN && *y!=d3_l->max_y-1) *y+=1;
  if(ke.key_id==DIKI_PAGE_UP && *z!=0) *z-=1;
  if(ke.key_id==DIKI_PAGE_DOWN && *z!=d3_l->max_z-1) *z+=1;
  if(ke.key_id==DIKI_Q) *q=1;
  if(ke.key_id==DIKI_ESCAPE) *esc=1;
  if(ke.key_id==DIKI_M) *switchmode=(*switchmode+1)%3;
  if(ke.key_id==DIKI_1) *bmp=1;
  if(ke.key_id==DIKI_2) *bmp=2;
  if(ke.key_id==DIKI_3) *bmp=3;
  if(ke.key_id==DIKI_4) *bmp=4;
  if(ke.key_id==DIKI_5) *bmp=5;
  if(ke.key_id==DIKI_A) *ac_distr=1;
  if(ke.key_id==DIKI_C) *ac_distr=2;
  if(ke.key_id==DIKI_B) *ac_distr=3;
 }

 return 1;
}

